// sFLC_Sample1.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include "conio.h"

int _tmain(int argc, _TCHAR* argv[])
{
	CFLC * FLC = new CFLC;		// instantiate a new FL 'empty' controller
	
	FLC->SetShape(0);			// Define a 3x3 FLC model

	int 	m_nDefaultFuzzySetModel = 0;
	int		m_nDefaultRuleSetModel = 0;

	FLC->SetRSModel(m_nDefaultRuleSetModel);

	int		ret = FLC->InitFuzzySets(3);
	if (ret < 0)
	{
		printf("Fuzzy Sets could not be created. Error:%i\n", ret);
		FLC->Reset();
		delete FLC;
		_getch();
		return -1;
	}
	else
		printf("\nNum empty FuzzySets created: %i", FLC->GetNumFuzzySets());

	int 	nNumFuzzyStates = 3;	// to be used in a 3x3 rule set
	int nNumFuzzySets = FLC->GetNumFuzzySets();
	CFuzzySet* pFSet = FLC->FSArray();

	for (int i = 0; i < nNumFuzzySets; i++)
	{
		int ret = pFSet[i].InitFuzzySet(nNumFuzzyStates,0);	
		if (ret < 0)
		{
			printf("\nFuzzy Sets could not be initialised. Error:%i\n", ret);
			FLC->Reset();
			delete FLC;
			_getch();
			return -1;
		}
		ret = pFSet[i].DefineFuzzySet(m_nDefaultFuzzySetModel); // before any possible change
		if (ret < 0)
		{
			printf("\nFuzzy Sets could not be defined. Error:%i\n", ret);
			FLC->Reset();
			delete FLC;
			_getch();
			return -1;
		}
		printf("\nFS%i is defined with model %i: %s", 
				i, m_nDefaultFuzzySetModel, pFSet[i].GetFSmodelname().c_str());
	}

	int nv;
	ret = FLC->InitVarSet(3);	// (default constructor for variables)
	if (ret < 0)
	{
		printf("\nVar Set could not be created: %i\n", ret);
		FLC->Reset();
		delete FLC;
		_getch();
		return -1;
	}
	else 
		printf("\n\n%i Vars created (1=Input,0=Output)", nv = FLC->GetNumVars());

	CFLVariable* pVars = FLC->VarArray();

	for (int i = 0; i < nv; i++)
	{	// Assuming here the last variable is the output
		BOOL bInputOutput = (i == nv - 1) ? FALSE : TRUE;
		int ret = pVars[i].LoadFuzzySet(pFSet[i],bInputOutput);
		if (ret < 0)
		{
			printf("\nFuzzy Set could not be linked to Var: %i. Error:%i\n", i, ret);
			FLC->Reset();
			delete FLC;
			_getch();
			return -1;
		}
		else
		{
			printf("\nVar%i (%i) has FS%i loaded (num states: %i)",
					i, bInputOutput, i,	pVars[i].NumStates()	);
		}
	}

	ret = FLC->DetermineRuleSetSize();
	if (ret < 0)
	{
		printf("\nRule Set could not be created: %i", ret);
		FLC->Reset();
		delete FLC;
		_getch();
		return -1;
	}
	else
		printf("\nRule Set size: %i \n", ret);

	int irs = FLC->InitRuleSet();	// generate an array of rules (RuleSetSize can be given or not)
	if (ret < 0)
	{
		printf("\nRule Set could not be initialised: %i\n", irs);
		FLC->Reset();
		delete FLC;
		_getch();
		return -1;
	}
	
	ret = FLC->AllocateRuleSpace();
	int NumRules = FLC->GetRuleSetSize();
	CFLRule *pFLR = FLC->RuleArray();	// get a pointer to the rule set

	int	RS_ret = FLC->LoadDefault33RuleSet(pFLR, NumRules,m_nDefaultRuleSetModel);
	if (RS_ret < 0)
	{
		printf("\nDefault Rule Set could not be loaded: %i", RS_ret);
		FLC->Reset();
		delete FLC;
		_getch();
		return -1;
	}
	else
		printf("\nRule Set follows model %i: '%s'",
		m_nDefaultRuleSetModel, FLC->RSModel().c_str());

	double ArrInput[2];
	ArrInput[0] = -0.25;
	ArrInput[1] = 0.25;	

	printf("\nInputs:\nVar1:\t%.2f\nVar2:\t%.2f\n\n",ArrInput[0], ArrInput[1]);

	FLC->Calculate(ArrInput);
//	FLC->EvaluateAllPremises(ArrInput);
//	FLC->InferenceRules();
//	FLC->GenerateFuzzyOutputs();
//	FLC->Defuzzify(0);

	int OutVarNo = FLC->GetNumVars() - 1; // last var is the output
	CFuzzySet *vFS = pVars[OutVarNo].FuzzySet();
	int NumOutputStates = vFS->NumStates();
	for (int i = 0; i < NumOutputStates; i++)
		printf("Fuzzy Output %i (%s): \t%.3f\n",
		i,
		vFS->FuzzyStateArray()[i].GetName().c_str(), 
		FLC->OutState(i));//s()[i]);

	double 	Output = FLC->Output();
	printf("\nDefuzzified Output: %.3f\n", Output);

	printf("\nPress a key to close");
	FLC->Reset();
	delete FLC;
	_getch();
	return 0;
}